@@ -41,8 +41,8 @@ enum {
 
 typedef struct ErrorItem {
     PipeItem base;
-    uint32_t reader_id;
-    uint32_t error;
+    VSCMsgHeader vheader;
+    VSCMsgError  error;
 } ErrorItem;
 
 typedef struct MsgItem {
@@ -288,32 +288,12 @@ static void smartcard_channel_send_data(RedChannel *channel, PipeItem *item, VSC
     red_channel_begin_send_message(channel);
 }
 
-static void smartcard_channel_send_message(RedChannel *channel, PipeItem *item,
-    uint32_t reader_id, VSCMsgType type, uint8_t* data, uint32_t len)
-{
-    VSCMsgHeader mhHeader;
-    //SpiceMarshaller* m = msg->marshaller();
-
-    mhHeader.type = type;
-    mhHeader.length = len;
-    mhHeader.reader_id = reader_id;
-    //_marshallers->msg_SpiceMsgData(m, &msgdata);
-    //spice_marshaller_add(m, (uint8_t*)&mhHeader, sizeof(mhHeader));
-    //spice_marshaller_add(m, data, len);
-    //marshaller_outgoing_write(msg);
-
-    smartcard_channel_send_data(channel, item, &mhHeader);
-}
-
 static void smartcard_channel_send_error(
     SmartCardChannel *smartcard_channel, PipeItem *item)
 {
     ErrorItem* error_item = (ErrorItem*)item;
-    VSCMsgError error;
 
-    error.code = error_item->error;
-    smartcard_channel_send_message(&smartcard_channel->base, item, error_item->reader_id,
-        VSC_Error, (uint8_t*)&error, sizeof(error));
+    smartcard_channel_send_data(&smartcard_channel->base, item, &error_item->vheader);
 }
 
 static void smartcard_channel_send_msg(
@@ -367,8 +347,10 @@ static void smartcard_push_error(SmartCardChannel* channel, uint32_t reader_id,
     ErrorItem *error_item = spice_new0(ErrorItem, 1);
 
     error_item->base.type = PIPE_ITEM_TYPE_ERROR;
-    error_item->reader_id = reader_id;
-    error_item->error = error;
+    error_item->vheader.reader_id = reader_id;
+    error_item->vheader.type = VSC_Error;
+    error_item->vheader.length = sizeof(error_item->error);
+    error_item->error.code = error;
     smartcard_channel_pipe_add(channel, &error_item->base);
 }
 
